@@ -32,6 +32,7 @@
 #include <stack>
 #include <unordered_set>
 
+class CanonicalForm;
 
 namespace GiNaC {
 #ifdef _MSC_VER
@@ -74,6 +75,8 @@ class symbol;
 struct symbolhasher;
 using symbolset = std::unordered_set<symbol,symbolhasher>;
 using expairvec = std::vector<std::pair<ex,ex>>;
+using ocvector = std::vector<numeric>;
+using power_ocvector_map = std::map<ex, ocvector, GiNaC::ex_is_less>;
 
 /** Lightweight wrapper for GiNaC's symbolic objects.  It holds a pointer to
  *  the other object in order to do garbage collection by the method of
@@ -100,7 +103,6 @@ public:
 	ex(int i);
 	ex(unsigned int i);
 	ex(long i);
-	ex(unsigned long i);
 	ex(double const d);
 	ex(PyObject* o);
 
@@ -134,7 +136,6 @@ public:
 	ex evalf(int level = 0, PyObject* parent=nullptr) const 
 	{ return bp->evalf(level, parent); }
 	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
-	ex eval_integ() const { return bp->eval_integ(); }
 
 	// printing
 	void print(const print_context & c, unsigned level = 0) const;
@@ -149,6 +150,7 @@ public:
 	size_t nsymbols() const;
         bool get_first_symbol(ex &x) const;
         symbolset symbols() const;
+        symbolset free_symbols() const;
 	const ex op(size_t i) const { return bp->op(i); }
 	ex sorted_op(size_t i) const;
 	ex operator[](const ex & index) const { return (*bp)[index]; }
@@ -201,6 +203,7 @@ public:
 	// differentiation and series expansion
 	ex diff(const symbol & s, unsigned nth = 1) const;
 	ex series(const ex & r, int order, unsigned options = 0) const;
+        void useries(flint_series_t& fp, int order) const { return bp->useries(fp, order); }
 
 	// rational functions
 	ex normal(int level = 0, bool noexpand_combined=false,
@@ -212,6 +215,9 @@ public:
 #ifdef PYNAC_HAVE_LIBGIAC
         const giac::polynome to_polynome(ex_int_map& map, exvector& revmap) const;
 #endif
+        const CanonicalForm to_canonical(ex_int_map& map,
+                        power_ocvector_map& pomap, exvector& revmap) const;
+        void collect_powers(power_ocvector_map& pomap) const;
 	ex numer() const;
 	ex denom() const;
 	ex numer_denom() const;
@@ -226,6 +232,8 @@ public:
 	void unitcontprim(const ex &x, ex &u, ex &c, ex &p) const;
 	ex smod(const numeric &xi) const { return bp->smod(xi); }
 	numeric max_coefficient() const;
+        bool is_linear(const symbol& x, ex& a, ex& b) const;
+        bool is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const;
 
 	// indexed objects
 	exvector get_free_indices() const { return bp->get_free_indices(); }
@@ -262,7 +270,6 @@ private:
 	static basic & construct_from_pyobject(PyObject* o);
 	static basic & construct_from_uint(unsigned int i);
 	static basic & construct_from_long(long i);
-	static basic & construct_from_ulong(unsigned long i);
 	static basic & construct_from_double(double d);
 	static ptr<basic> construct_from_string_and_lst(const std::string &s, const ex &l);
 	void makewriteable();
@@ -322,12 +329,6 @@ ex::ex(long i) : bp(construct_from_long(i))
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
 
-inline
-ex::ex(unsigned long i) : bp(construct_from_ulong(i))
-{
-	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-}
-
 inline
 ex::ex(double const d) : bp(construct_from_double(d))
 {
@@ -752,9 +753,6 @@ inline ex eval(const ex & thisex, int level = 0)
 inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=nullptr)
 { return thisex.evalf(level, parent); }
 
-inline ex eval_integ(const ex & thisex)
-{ return thisex.eval_integ(); }
-
 inline ex diff(const ex & thisex, const symbol & s, unsigned nth = 1)
 { return thisex.diff(s, nth); }
 
